# GnuPGverify.pm
#
# A module for checking GnuPG signatures.
#
# author:  Max-Gerd Retzlaff <m.retzlaff@gmx.net>
#
#          parts of the GnuPG-Code in check_signature()
#             by Reyk Floeter <reyk@synack.de>
#
# date:    $Date: 2002/04/26 16:14:06 $
# version: $Revision: 1.5 $
#
# This application is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version. 
#
# This application is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# 02111 USA.  

package GnuPGverify;

use HelpfulFunctions; 

use GnuPG::Interface;
use IO::Handle;

if ( defined(%ENV) && exists($ENV{HOME}) ) {
    $home_dir = "$ENV{HOME}";
} else {
    $home_dir = "/tmp";
}
$gnupg_dir = "$home_dir/.gnupg";
$passphrase = undef;

if (! -d $gnupg_dir) {
	mdie(0, "ERROR: No GnuPG directory: $gnupg_dir");
}

sub set_gnupg_dir {
    my ($pkg, $dir) = @_;
    $gnupg_dir = $dir;
}

sub set_home_dir {
    my ($pkg, $dir) = @_;
    $home_dir = $dir;
}

sub set_passphrase {
    my ($pkg, $pass) = @_;
    $passphrase = $pass;
}

sub check_signature {
    my ($pkg, $alias, $content_part, $gpg_signature) = @_[0..3];
    my %sections = %{$_[4]};

    use File::Temp;

    my $gnupg = GnuPG::Interface->new();
    $gnupg->options->hash_init(armor => 1, 
			       textmode => 1,
			       homedir => $gnupg_dir);
    $gnupg->options->meta_interactive(0); # suppress some babbling...

    my $input = IO::Handle->new();
    my $error = IO::Handle->new();
    my $output = IO::Handle->new();
    my $handles = GnuPG::Handles->new(stdin => $input, 
				      stderr => $error,
				      stdout => $output);

    my ($sigfh, $sigfile) = 
	File::Temp::tempfile('/tmp/mwiki-sigXXXXX', UNLINK => 1);
    print $sigfh $gpg_signature;

    my $pid = $gnupg->verify(handles => $handles,  
			     command_args => [ "$sigfile", "-" ] );

    print $input $content_part;
    close $sigfh;
    close $input;
    close $output;

    my @out = <$error>;
    close $error;

    waitpid $pid, 0;  # clean up the finished GnuPG process
    
    # print @out,"\n";

    my $sign_ID;
    if ($out[1] =~ /^gpg: Good signature/i) {
	$sign_ID = $out[0];
	$sign_ID =~ s/^.*?\sID\s([^\s]*)$/0x$1/gi;
	chomp $sign_ID;
    } else {
	mdie(0, "ERROR: gpg: BAD signature, " . $out[1] . "\n");
	return 1;
    }

    foreach my $valid_section (sort(keys %sections)) {
	# if ( $valid_section !~ /^MWIKICONFFILE_/ ) {
	    # we can use section names as well
	    foreach my $valid_ID (@{$sections{$valid_section}->{add_IDs}},
				  @{$sections{$valid_section}->{adm_IDs}}) {
		if ( $valid_ID eq $sign_ID ) {
			return(0, $sign_ID); # access granted
		}
	    }
	    # aliases
	    foreach my $valid_alias (@{$sections{$valid_section}->{aliases}} ) {
		if ($alias eq $valid_alias) {
		    foreach my $valid_ID (@{$sections{$valid_section}->{add_IDs}},
					  @{$sections{$valid_section}->{adm_IDs}}) {
			if ( $valid_ID eq $sign_ID ) {
			    return (0, $sign_ID); # access granted
			}
		    }
		}
	    }
	# }
    }
    
    mdie(0, "ERROR: access denied for $sign_ID (for messages of alias $alias)\n");
    return 2;
}

sub create_signature{
    my ($pkg, $content_part) = @_;

    mdie(1, "ERROR: Don't know your passphrase.\n") if (!defined $passphrase);

    my $gnupg = GnuPG::Interface->new();
    $gnupg->options->hash_init(armor => 1, 
			       textmode => 1,
			       homedir => $gnupg_dir);
    $gnupg->options->meta_interactive(0); # suppress some babbling...

    my $input = IO::Handle->new();
    my $error = IO::Handle->new();
    my $output = IO::Handle->new();
    my $handles = GnuPG::Handles->new(stdin => $input, 
				      stderr => $error,
				      stdout => $output);

    $gnupg->passphrase( $passphrase );

    my $pid = $gnupg->detach_sign( handles => $handles );

    print $input $content_part;
    close $input;
    my @out = <$output>;
    close $output;
    close $error;

    waitpid $pid, 0;  # clean up the finished GnuPG process

    my $signature = join('', @out);

    mdie(1, "ERROR: Couldn't create signature.\n") if (!defined $signature);

    return (0, $signature );
}

sub export_keys {

    my $gnupg = GnuPG::Interface->new();
    $gnupg->options->hash_init(armor => 1, 
			       textmode => 1,
			       homedir => $gnupg_dir);
    $gnupg->options->meta_interactive(0); # suppress some babbling...

    my $input = IO::Handle->new();
    my $error = IO::Handle->new();
    my $output = IO::Handle->new();
    my $handles = GnuPG::Handles->new(stdin => $input, 
				      stderr => $error,
				      stdout => $output);

    my $pid = $gnupg->export_keys(handles => $handles );

    close $input;
    close $error;
    
    my @out = <$output>;
    close $output;

    waitpid $pid, 0;  # clean up the finished GnuPG process
    
    my $keys = join('', @out);

    mdie(1, "ERROR: Couldn't export the keys.\n") if (!defined $keys);

    return (0, $keys );
}

sub import_keys {
    my ($pkg, $content_part) = @_;

    my $gnupg = GnuPG::Interface->new();
    $gnupg->options->hash_init(armor => 1, 
			       textmode => 1,
			       homedir => $gnupg_dir);
    $gnupg->options->meta_interactive(0); # suppress some babbling...

    my $input = IO::Handle->new();
    my $error = IO::Handle->new();
    my $output = IO::Handle->new();
    my $handles = GnuPG::Handles->new(stdin => $input, 
				      stderr => $error,
				      stdout => $output);

    my $pid = $gnupg->import_keys(handles => $handles );

    print $input $content_part;
    close $input;
    close $error;
    close $output;

    waitpid $pid, 0;  # clean up the finished GnuPG process
    
    return 0;
}

return 1; # Last executing statement in file must be
          # non-zero, to indicate succesful loading
